#ifndef GOOGLE_UPB_UPB_GENERATOR_PLUGIN_BOOTSTRAP_H__
#define GOOGLE_UPB_UPB_GENERATOR_PLUGIN_BOOTSTRAP_H__

// IWYU pragma: begin_exports

#if defined(UPB_BOOTSTRAP_STAGE) && UPB_BOOTSTRAP_STAGE == 0
// This header is checked in.
#include "upb_generator/stage0/google/protobuf/compiler/plugin.upb.h"
#elif defined(UPB_BOOTSTRAP_STAGE) && UPB_BOOTSTRAP_STAGE == 1
// This header is generated at build time by the bootstrapping process.
#include "upb_generator/stage1/google/protobuf/compiler/plugin.upb.h"
#else
// This is the normal header, generated by upb_c_proto_library().
#include "google/protobuf/compiler/plugin.upb.h"
#endif

// IWYU pragma: end_exports

#endif  // GOOGLE_UPB_UPB_GENERATOR_PLUGIN_BOOTSTRAP_H__
